@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2013-2016 Aerospike, Inc.
+ * Copyright 2013-2017 Aerospike, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -25,6 +25,7 @@
 #include <aerospike/as_record.h>
 #include <aerospike/as_udf.h>
 #include <aerospike/as_operations.h>
+#include <aerospike/aerospike_batch.h>
 
 #include "types.h"
 
@@ -38,7 +39,7 @@ as_status as_user_to_pyobject(as_error *err, as_user *user, PyObject **py_as_use
 
 as_status as_user_array_to_pyobject(as_error *err, as_user **users, PyObject **py_as_users, int users_size);
 
-as_status pyobject_to_strArray(as_error * err, PyObject * py_list,  char **arr);
+as_status pyobject_to_strArray(as_error * err, PyObject * py_list,  char **arr, uint32_t max_len);
 
 as_status pyobject_to_val(AerospikeClient * self, as_error * err, PyObject * py_obj, as_val ** val, as_static_pool *static_pool, int serializer_type);
 
@@ -65,6 +66,8 @@ as_status as_list_of_map_to_py_tuple_list(AerospikeClient * self, as_error * err
 
 as_status record_to_pyobject(AerospikeClient * self, as_error * err, const as_record * rec, const as_key * key, PyObject ** obj);
 
+as_status operate_bins_to_pyobject(AerospikeClient * self, as_error * err, const as_record * rec, PyObject ** py_bins);
+
 as_status record_to_pyobject_cnvt_list_to_map(AerospikeClient * self, as_error * err, const as_record * rec, const as_key * key, PyObject ** obj);
 
 as_status key_to_pyobject(as_error * err, const as_key * key, PyObject ** obj);
@@ -75,8 +78,6 @@ as_status bins_to_pyobject(AerospikeClient * self, as_error * err, const as_reco
 
 bool error_to_pyobject(const as_error * err, PyObject ** obj);
 
-as_status initialize_ldt(as_error *error, as_ldt* ldt_p, char* bin_name, int type, char* module);
-
 as_status pyobject_to_astype_write(AerospikeClient * self, as_error * err, PyObject * py_value, as_val **val,
 	as_static_pool *static_pool, int serializer_type);
 
@@ -93,3 +94,11 @@ void initialize_bin_for_strictypes(AerospikeClient *self, as_error *err, PyObjec
 as_status bin_strict_type_checking(AerospikeClient * self, as_error *err, PyObject *py_bin, char **bin);
 
 as_status check_for_meta(PyObject * py_meta, as_operations * ops, as_error *err);
+
+as_status as_batch_read_results_to_pyobject(as_error* err, AerospikeClient* client, const as_batch_read* results,
+		uint32_t size, PyObject** py_records);
+
+as_status batch_read_records_to_pyobject(AerospikeClient *self, as_error *err, as_batch_read_records* records, PyObject **py_recs);
+
+as_status
+string_and_pyuni_from_pystring(PyObject* py_string, PyObject** pyuni_r, char** c_str_ptr, as_error* err);
\ No newline at end of file
