@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2013-2016 Aerospike, Inc.
+ * Copyright 2013-2017 Aerospike, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,9 +16,11 @@
 
 #include <Python.h>
 #include <aerospike/as_error.h>
+#include <aerospike/as_query.h>
 #include <aerospike/as_policy.h>
 #include <aerospike/as_policy.h>
 #include <aerospike/as_map_operations.h>
+#include <aerospike/as_list_operations.h>
 
 #define MAX_CONSTANT_STR_SIZE 512
 
@@ -49,7 +51,26 @@ enum Aerospike_list_operations {
 	OP_LIST_GET,
 	OP_LIST_GET_RANGE,
 	OP_LIST_TRIM,
-	OP_LIST_SIZE
+	OP_LIST_SIZE,
+	OP_LIST_INCREMENT,
+	OP_LIST_GET_BY_INDEX,
+	OP_LIST_GET_BY_INDEX_RANGE,
+	OP_LIST_GET_BY_RANK,
+	OP_LIST_GET_BY_RANK_RANGE,
+	OP_LIST_GET_BY_VALUE,
+	OP_LIST_GET_BY_VALUE_LIST,
+	OP_LIST_GET_BY_VALUE_RANGE,
+	OP_LIST_REMOVE_BY_INDEX,
+	OP_LIST_REMOVE_BY_INDEX_RANGE,
+	OP_LIST_REMOVE_BY_RANK,
+	OP_LIST_REMOVE_BY_RANK_RANGE,
+	OP_LIST_REMOVE_BY_VALUE,
+	OP_LIST_REMOVE_BY_VALUE_LIST,
+	OP_LIST_REMOVE_BY_VALUE_RANGE,
+	OP_LIST_SET_ORDER,
+	OP_LIST_SORT,
+	OP_LIST_REMOVE_BY_VALUE_RANK_RANGE_REL,
+	OP_LIST_GET_BY_VALUE_RANK_RANGE_REL
 };
 
 enum Aerospike_map_operations {
@@ -78,6 +99,12 @@ enum Aerospike_map_operations {
 	OP_MAP_GET_BY_INDEX_RANGE,
 	OP_MAP_GET_BY_RANK,
 	OP_MAP_GET_BY_RANK_RANGE,
+	OP_MAP_GET_BY_VALUE_LIST,
+	OP_MAP_GET_BY_KEY_LIST,
+	OP_MAP_REMOVE_BY_VALUE_RANK_RANGE_REL,
+	OP_MAP_REMOVE_BY_KEY_INDEX_RANGE_REL,
+	OP_MAP_GET_BY_VALUE_RANK_RANGE_REL,
+	OP_MAP_GET_BY_KEY_INDEX_RANGE_REL
 };
 
 typedef struct Aerospike_Constants {
@@ -148,3 +175,9 @@ as_status pyobject_to_map_policy(as_error * err, PyObject * py_policy,
 as_status declare_policy_constants(PyObject *aerospike);
 
 void set_scan_options(as_error *err, as_scan* scan_p, PyObject * py_options);
+
+as_status set_query_options(as_error* err, PyObject* query_options, as_query* query);
+
+as_status pyobject_to_list_policy(as_error * err, PyObject * py_policy,
+									as_list_policy * policy);
+
