@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2013-2016 Aerospike, Inc.
+ * Copyright 2013-2017 Aerospike, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -90,9 +90,12 @@ PyObject * AerospikeClient_Admin_Create_User(AerospikeClient * self, PyObject *a
 			roles[i] = cf_malloc(sizeof(char) * AS_ROLE_SIZE);
 			memset(roles[i], 0, sizeof(char) * AS_ROLE_SIZE);
 		}
+	} else {
+		as_error_update(&err, AEROSPIKE_ERR_PARAM, "Roles should be a list");
+		goto CLEANUP;
 	}
 
-	pyobject_to_strArray(&err, py_roles, roles);
+	pyobject_to_strArray(&err, py_roles, roles, AS_ROLE_SIZE);
 	if (err.code != AEROSPIKE_OK) {
 		goto CLEANUP;
 	}
@@ -507,7 +510,7 @@ PyObject * AerospikeClient_Admin_Grant_Roles( AerospikeClient *self, PyObject *a
 		}
 	}
 
-	pyobject_to_strArray(&err, py_roles, roles);
+	pyobject_to_strArray(&err, py_roles, roles, AS_ROLE_SIZE);
 	if (err.code != AEROSPIKE_OK) {
 		goto CLEANUP;
 	}
@@ -615,7 +618,7 @@ PyObject * AerospikeClient_Admin_Revoke_Roles( AerospikeClient *self, PyObject *
 		}
 	}
 
-	pyobject_to_strArray(&err, py_roles, roles);
+	pyobject_to_strArray(&err, py_roles, roles, AS_ROLE_SIZE);
 	if (err.code != AEROSPIKE_OK) {
 		goto CLEANUP;
 	}
@@ -827,7 +830,7 @@ PyObject * AerospikeClient_Admin_Query_Users( AerospikeClient * self, PyObject *
 	aerospike_query_users(self->as, &err, admin_policy_p, &users, &users_size);
 	Py_END_ALLOW_THREADS
 	if (err.code != AEROSPIKE_OK) {
-		as_error_update(&err, err.code, err.message);
+		as_error_update(&err, err.code, NULL);
 		goto CLEANUP;
 	}
 
