@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2013-2016 Aerospike, Inc.
+ * Copyright 2013-2017 Aerospike, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -29,6 +29,14 @@
 #include "exceptions.h"
 #include "policy.h"
 
+
+typedef struct _exists_many_cb_data {
+	PyObject* py_recs;
+	as_error* cb_err;
+}exists_many_cb_data;
+
+static void
+make_batch_safe_to_free(as_batch* batch, int size);
 /**
  *******************************************************************************************************
  * This callback will be called with the results with aerospike_batch_exists().
@@ -45,258 +53,69 @@ static
 bool batch_exists_cb(const as_batch_read* results, uint32_t n, void* udata)
 {
 	// Typecast udata back to PyObject
-	PyObject * py_recs = (PyObject *) udata;
+	exists_many_cb_data* local_data = (exists_many_cb_data*)udata;
+	PyObject * py_recs = local_data->py_recs;
+	as_error local_err;
+	as_error_init(&local_err);
 
 	// Lock Python State
 	PyGILState_STATE gstate;
 	gstate = PyGILState_Ensure();
 
 	// Loop over results array
-	for (uint32_t i =0; i < n; i++) {
+	for (uint32_t i = 0; i < n; i++) {
 
-		PyObject * rec = PyDict_New();
 		PyObject * py_rec = NULL;
-		PyObject * p_key = NULL;
-		py_rec = PyTuple_New(2);
-		p_key = PyTuple_New(4);
+		PyObject* py_meta = NULL;
+		PyObject* py_key = NULL;
 
-		if (results[i].key->ns && strlen(results[i].key->ns) > 0) {
-			PyTuple_SetItem(p_key, 0, PyString_FromString(results[i].key->ns));
-		}
+		if (results[i].result == AEROSPIKE_OK) {
+			key_to_pyobject(&local_err, results[i].key, &py_key);
+			if (!py_key) {
+				py_key = Py_None;
+				Py_INCREF(py_key);
+			}
 
-		if (results[i].key->set && strlen(results[i].key->set) > 0) {
-			PyTuple_SetItem(p_key, 1, PyString_FromString(results[i].key->set));
-		}
-
-		if (results[i].key->valuep) {
-			switch(((as_val*)(results[i].key->valuep))->type) {
-				case AS_INTEGER:
-					PyTuple_SetItem(p_key, 2, PyInt_FromLong((long)results[i].key->value.integer.value));
-					break;
-
-				case AS_STRING:
-					PyTuple_SetItem(p_key, 2, PyString_FromString((const char *)results[i].key->value.string.value));
-					break;
-				default:
-					break;
+			metadata_to_pyobject(&local_err, &results[i].record, &py_meta);
+			if (!py_meta) {
+				py_meta = Py_None;
+				Py_INCREF(py_meta);
+			}
+			py_rec = Py_BuildValue("OO", py_key, py_meta);
+			Py_DECREF(py_key);
+			Py_DECREF(py_meta);
+			if (!py_rec) {
+				as_error_update(local_data->cb_err, AEROSPIKE_ERR_CLIENT, "Failed to create metadata tuple");
+				PyGILState_Release(gstate);
+				return false;
 			}
 		} else {
-			Py_INCREF(Py_None);
-			PyTuple_SetItem(p_key, 2, Py_None);
-		}
-		if (results[i].key->digest.init) {
-			PyTuple_SetItem(p_key, 3, PyByteArray_FromStringAndSize((char *) results[i].key->digest.value, AS_DIGEST_VALUE_SIZE));
-		}
-
-		PyTuple_SetItem(py_rec, 0, p_key);
-		if (results[i].result == AEROSPIKE_OK) {
-			PyObject *py_gen = PyInt_FromLong((long)results[i].record.gen);
-			PyDict_SetItemString( rec, "gen", py_gen );
-			Py_DECREF(py_gen);
-			PyObject *py_ttl = PyInt_FromLong((long)results[i].record.ttl);
-			PyDict_SetItemString( rec, "ttl", py_ttl );
-			Py_DECREF(py_ttl);
-
-			PyTuple_SetItem(py_rec, 1, rec);
-			if (PyList_SetItem( py_recs, i, py_rec )) {
-				// Release Python State
-				PyGILState_Release(gstate);
-				return false;
+			key_to_pyobject(&local_err, results[i].key, &py_key);
+			if (!py_key) {
+				py_key= Py_None;
+				Py_INCREF(py_key);
 			}
-		} else if (results[i].result == AEROSPIKE_ERR_RECORD_NOT_FOUND) {
-			Py_DECREF(rec);
-			Py_INCREF(Py_None);
-			PyTuple_SetItem(py_rec, 1, Py_None);
-
-			if (PyList_SetItem( py_recs, i, py_rec)) {
-				// Release Python State
+			py_rec = Py_BuildValue("OO", py_key, Py_None);
+			Py_DECREF(py_key);
+			if (!py_rec) {
 				PyGILState_Release(gstate);
+				as_error_update(local_data->cb_err, AEROSPIKE_ERR_CLIENT, "Failed to create metadata tuple");
 				return false;
 			}
 		}
+		if (PyList_SetItem(py_recs, i, py_rec) != 0) {
+			Py_XDECREF(py_rec);
+			PyGILState_Release(gstate);
+			as_error_update(local_data->cb_err, AEROSPIKE_ERR_CLIENT, "Failed to add record to metadata tuple");
+			return false;
+		}
+
 	}
 	// Release Python State
 	PyGILState_Release(gstate);
 	return true;
 }
 
-/**
- *******************************************************************************************************
- * This callback will be called with the results with aerospike_batch_exists().
- *
- * @param err                   Error object
- * @param records               An array of as_batch_read_record entries
- * @param py_recs               The pyobject to be filled in with the return
- *                              value
- *
- * Returns boolean value(true or false).
- *******************************************************************************************************
- */
-static
-void batch_exists_recs(as_error *err, as_batch_read_records* records, PyObject **py_recs)
-{
-	// Loop over records array
-	as_vector* list = &records->list;
-	for (uint32_t i = 0; i < list->size; i++) {
-		as_batch_read_record* batch = as_vector_get(list, i);
-
-		PyObject * rec = PyDict_New();
-		PyObject * p_key = NULL;
-		PyObject * py_rec = NULL;
-		py_rec = PyTuple_New(2);
-		p_key = PyTuple_New(4);
-
-		if (batch->key.ns && strlen(batch->key.ns) > 0) {
-			PyTuple_SetItem(p_key, 0, PyString_FromString(batch->key.ns));
-		}
-
-		if (batch->key.set && strlen(batch->key.set) > 0) {
-			PyTuple_SetItem(p_key, 1, PyString_FromString(batch->key.set));
-		}
-
-		if (batch->key.valuep) {
-			switch(((as_val*)(batch->key.valuep))->type) {
-				case AS_INTEGER:
-					PyTuple_SetItem(p_key, 2, PyInt_FromLong((long)batch->key.value.integer.value));
-					break;
-
-				case AS_STRING:
-					PyTuple_SetItem(p_key, 2, PyString_FromString((const char *)batch->key.value.string.value));
-					break;
-				default:
-					break;
-			}
-		} else {
-			Py_INCREF(Py_None);
-			PyTuple_SetItem(p_key, 2, Py_None);
-		}
-
-		if (batch->key.digest.init) {
-			PyTuple_SetItem(p_key, 3, PyByteArray_FromStringAndSize((char *) batch->key.digest.value, AS_DIGEST_VALUE_SIZE));
-		}
-
-		PyTuple_SetItem(py_rec, 0, p_key);
-		if (batch->result == AEROSPIKE_OK) {
-
-			PyObject *py_gen = PyInt_FromLong((long)batch->record.gen);
-			PyDict_SetItemString( rec, "gen", py_gen );
-			Py_DECREF(py_gen);
-			PyObject *py_ttl = PyInt_FromLong((long)batch->record.ttl);
-			PyDict_SetItemString( rec, "ttl", py_ttl );
-			Py_DECREF(py_ttl);
-
-			PyTuple_SetItem(py_rec, 1, rec);
-			PyList_SetItem( *py_recs, i, py_rec );
-		} else if (batch->result == AEROSPIKE_ERR_RECORD_NOT_FOUND) {
-			Py_DECREF(rec);
-			Py_INCREF(Py_None);
-			PyTuple_SetItem(py_rec, 1, Py_None);
-			PyList_SetItem( *py_recs, i, py_rec);
-		}
-	}
-}
-/**
- *******************************************************************************************************
- * This function will get a batch of records from the Aeropike DB.
- *
- * @param err                   as_error object
- * @param self                  AerospikeClient object
- * @param py_keys               The list of keys
- * @param batch_policy_p        as_policy_batch object
- *
- * Returns the record if key exists otherwise NULL.
- *******************************************************************************************************
- */
-static PyObject * batch_exists_aerospike_batch_read(as_error *err, AerospikeClient * self, PyObject *py_keys, as_policy_batch * batch_policy_p)
-{
-	PyObject * py_recs = NULL;
-
-	as_batch_read_records records;
-
-	as_batch_read_record* record = NULL;
-	bool batch_initialised = false;
-
-	// Convert python keys list to as_key ** and add it to as_batch.keys
-	// keys can be specified in PyList or PyTuple
-	if (py_keys && PyList_Check(py_keys)) {
-		Py_ssize_t size = PyList_Size(py_keys);
-
-		py_recs = PyList_New(size);
-		as_batch_read_inita(&records, size);
-
-		// Batch object initialised
-		batch_initialised = true;
-
-		for (int i = 0; i < size; i++) {
-
-			PyObject * py_key = PyList_GetItem(py_keys, i);
-
-			if (!PyTuple_Check(py_key)) {
-				as_error_update(err, AEROSPIKE_ERR_PARAM, "Key should be a tuple.");
-				goto CLEANUP;
-			}
-
-			record = as_batch_read_reserve(&records);
-
-			pyobject_to_key(err, py_key, &record->key);
-			record->read_all_bins = true;
-
-			if (err->code != AEROSPIKE_OK) {
-				goto CLEANUP;
-			}
-		}
-	}
-	else if (py_keys && PyTuple_Check(py_keys)) {
-		Py_ssize_t size = PyTuple_Size(py_keys);
-
-		py_recs = PyList_New(size);
-		as_batch_read_inita(&records, size);
-		// Batch object initialised
-		batch_initialised = true;
-
-		for (int i = 0; i < size; i++) {
-			PyObject * py_key = PyTuple_GetItem(py_keys, i);
-
-			if (!PyTuple_Check(py_key)) {
-				as_error_update(err, AEROSPIKE_ERR_PARAM, "Key should be a tuple.");
-				goto CLEANUP;
-			}
-
-			record = as_batch_read_reserve(&records);
-
-			pyobject_to_key(err, py_key, &record->key);
-			record->read_all_bins = true;
-
-			if (err->code != AEROSPIKE_OK) {
-				goto CLEANUP;
-			}
-		}
-	}
-	else {
-		as_error_update(err, AEROSPIKE_ERR_PARAM, "Keys should be specified as a list or tuple.");
-		goto CLEANUP;
-	}
-
-	// Invoke C-client API
-	Py_BEGIN_ALLOW_THREADS
-	aerospike_batch_read(self->as, err, batch_policy_p, &records);
-	Py_END_ALLOW_THREADS
-	if (err->code != AEROSPIKE_OK) {
-		goto CLEANUP;
-	}
-	batch_exists_recs(err, &records, &py_recs);
-
-CLEANUP:
-	if (batch_initialised == true) {
-		// We should destroy batch object as we are using 'as_batch_init' for initialisation
-		// Also, pyobject_to_key is soing strdup() in case of Unicode. So, object destruction
-		// is necessary.
-		as_batch_read_destroy(&records);
-	}
-
-	return py_recs;
-}
-
 /**
  *******************************************************************************************************
  * This function will get a batch of records from the Aeropike DB.
@@ -311,10 +130,14 @@ CLEANUP:
  */
 static PyObject * batch_exists_aerospike_batch_exists(as_error *err, AerospikeClient * self, PyObject *py_keys, as_policy_batch * batch_policy_p)
 {
-	PyObject * py_recs = NULL;
 
 	as_batch batch;
 	bool batch_initialised = false;
+	exists_many_cb_data cb_data;
+	as_error local_err;
+	as_error_init(&local_err);
+	cb_data.py_recs = NULL;
+	cb_data.cb_err = &local_err;
 
 	// Convert python keys list to as_key ** and add it to as_batch.keys
 	// keys can be specified in PyList or PyTuple
@@ -322,8 +145,13 @@ static PyObject * batch_exists_aerospike_batch_exists(as_error *err, AerospikeCl
 		Py_ssize_t size = PyList_Size(py_keys);
 
 		as_batch_init(&batch, size);
+		make_batch_safe_to_free(&batch, size);
 
-		py_recs = PyList_New(size);
+		cb_data.py_recs = PyList_New(size);
+		if (!cb_data.py_recs) {
+			as_error_update(err, AEROSPIKE_ERR_PARAM, "Failed to allocate return record");
+			goto CLEANUP;
+		}
 		// Batch object initialised
 		batch_initialised = true;
 
@@ -346,10 +174,16 @@ static PyObject * batch_exists_aerospike_batch_exists(as_error *err, AerospikeCl
 	else if (py_keys && PyTuple_Check(py_keys)) {
 		Py_ssize_t size = PyTuple_Size(py_keys);
 
-		py_recs = PyList_New(size);
+		cb_data.py_recs = PyList_New(size);
+		if (!cb_data.py_recs) {
+			as_error_update(err, AEROSPIKE_ERR_PARAM, "Failed to allocate return record");
+			goto CLEANUP;
+		}
+
 		as_batch_init(&batch, size);
 		// Batch object initialised
 		batch_initialised = true;
+		make_batch_safe_to_free(&batch, size);
 
 		for (int i = 0; i < size; i++) {
 			PyObject * py_key = PyTuple_GetItem(py_keys, i);
@@ -374,10 +208,15 @@ static PyObject * batch_exists_aerospike_batch_exists(as_error *err, AerospikeCl
 	// Invoke C-client API
 	Py_BEGIN_ALLOW_THREADS
 	aerospike_batch_exists(self->as, err, batch_policy_p, &batch,
-			(aerospike_batch_read_callback) batch_exists_cb, py_recs);
+			(aerospike_batch_read_callback) batch_exists_cb, &cb_data);
 	Py_END_ALLOW_THREADS
 	if (err->code != AEROSPIKE_OK) {
 		as_error_update(err, err->code, NULL);
+		Py_CLEAR(cb_data.py_recs);
+	}
+	if (cb_data.cb_err->code != AEROSPIKE_OK) {
+		as_error_update(err, cb_data.cb_err->code, cb_data.cb_err->message);
+		Py_CLEAR(cb_data.py_recs);
 	}
 
 CLEANUP:
@@ -388,7 +227,7 @@ CLEANUP:
 		as_batch_destroy(&batch);
 	}
 
-	return py_recs;
+	return cb_data.py_recs;
 }
 /**
  *******************************************************************************************************
@@ -413,8 +252,6 @@ PyObject * AerospikeClient_Exists_Many_Invoke(
 	as_error err;
 	as_policy_batch policy;
 	as_policy_batch * batch_policy_p = NULL;
-	bool has_batch_index = false;
-
 	// Initialize error
 	as_error_init(&err);
 
@@ -435,16 +272,9 @@ PyObject * AerospikeClient_Exists_Many_Invoke(
 		goto CLEANUP;
 	}
 
-	has_batch_index = aerospike_has_batch_index(self->as);
+	py_recs = batch_exists_aerospike_batch_exists(&err, self, py_keys,
+			batch_policy_p);
 
-	if (has_batch_index
-			&& !(self->as->config.policies.batch.use_batch_direct)) {
-		py_recs = batch_exists_aerospike_batch_read(&err, self, py_keys,
-				batch_policy_p);
-	} else {
-		py_recs = batch_exists_aerospike_batch_exists(&err, self, py_keys,
-				batch_policy_p);
-	}
 
 CLEANUP:
 
@@ -498,3 +328,20 @@ PyObject * AerospikeClient_Exists_Many(AerospikeClient * self, PyObject * args,
 	// Invoke Operation
 	return AerospikeClient_Exists_Many_Invoke(self, py_keys, py_policy);
 }
+
+/*
+ * This marks each key in the batch's value pointer as null
+ * and sets it to not be freed on as_key_destroy.
+ * This is needed so that as_batch_destroy does not try to free
+ * any uninitialized data.
+ */
+static void
+make_batch_safe_to_free(as_batch* batch, int size) {
+	for (int i = 0; i < size; i++) {
+		as_key* batch_key = as_batch_keyat(batch, i);
+		if (batch_key) {
+			batch_key->valuep = NULL;
+			batch_key->_free = false;
+		}
+	}
+}
