@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2013-2016 Aerospike, Inc.
+ * Copyright 2013-2017 Aerospike, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -23,6 +23,7 @@
 #include <aerospike/as_error.h>
 #include <aerospike/as_record.h>
 #include <aerospike/as_operations.h>
+#include <aerospike/as_map_operations.h>
 #include <aerospike/aerospike_info.h>
 #include "client.h"
 #include "conversions.h"
@@ -30,6 +31,8 @@
 #include "policy.h"
 #include "serializer.h"
 #include "geo.h"
+#include "cdt_list_operations.h"
+#include "cdt_map_operations.h"
 
 #include <aerospike/as_double.h>
 #include <aerospike/as_integer.h>
@@ -37,6 +40,15 @@
 #include <aerospike/as_nil.h>
 
 
+
+static as_status
+get_operation(as_error* err, PyObject* op_dict, long* operation_ptr);
+
+static inline bool isListOp(int op);
+static inline bool isNewMapOp(int op);
+
+#define PY_OPERATION_KEY "op"
+
 #define BASE_VARIABLES\
 	as_error err;\
 	as_error_init(&err);\
@@ -102,6 +114,8 @@
 			static_pool, SERIALIZER_PYTHON) != AEROSPIKE_OK) {\
 		return err->code;\
 	}
+
+static as_status invertIfSpecified(as_error* err, PyObject* op_dict, uint64_t* return_value);
 /**
  *******************************************************************************************************
  * This function will check whether operation can be performed
@@ -167,6 +181,49 @@ int check_type(AerospikeClient * self, PyObject * py_value, int op, as_error *er
 	return 0;
 }
 
+static inline bool isListOp(int op) {
+	return (op == OP_LIST_APPEND ||
+			op == OP_LIST_APPEND_ITEMS ||
+			op == OP_LIST_INSERT ||
+			op == OP_LIST_INSERT_ITEMS ||
+			op == OP_LIST_POP ||
+			op == OP_LIST_POP_RANGE ||
+			op == OP_LIST_REMOVE ||
+			op == OP_LIST_REMOVE_RANGE ||
+			op == OP_LIST_CLEAR ||
+			op == OP_LIST_SET ||
+			op == OP_LIST_GET ||
+			op == OP_LIST_GET_RANGE ||
+			op == OP_LIST_TRIM ||
+			op == OP_LIST_SIZE ||
+			op == OP_LIST_INCREMENT ||
+			op == OP_LIST_GET_BY_INDEX ||
+			op == OP_LIST_GET_BY_INDEX_RANGE ||
+			op == OP_LIST_GET_BY_RANK ||
+			op == OP_LIST_GET_BY_RANK_RANGE ||
+			op == OP_LIST_GET_BY_VALUE ||
+			op == OP_LIST_GET_BY_VALUE_LIST ||
+			op == OP_LIST_GET_BY_VALUE_RANGE ||
+			op == OP_LIST_REMOVE_BY_INDEX ||
+			op == OP_LIST_REMOVE_BY_INDEX_RANGE ||
+			op == OP_LIST_REMOVE_BY_RANK ||
+			op == OP_LIST_REMOVE_BY_RANK_RANGE ||
+			op == OP_LIST_REMOVE_BY_VALUE ||
+			op == OP_LIST_REMOVE_BY_VALUE_LIST ||
+			op == OP_LIST_REMOVE_BY_VALUE_RANGE ||
+			op == OP_LIST_SET_ORDER||
+			op == OP_LIST_SORT ||
+			op == OP_LIST_REMOVE_BY_VALUE_RANK_RANGE_REL ||
+			op == OP_LIST_GET_BY_VALUE_RANK_RANGE_REL);
+}
+
+static inline bool isNewMapOp(int op) {
+	return (op == OP_MAP_REMOVE_BY_KEY_INDEX_RANGE_REL ||
+			op == OP_MAP_REMOVE_BY_VALUE_RANK_RANGE_REL ||
+			op == OP_MAP_GET_BY_VALUE_RANK_RANGE_REL ||
+			op == OP_MAP_GET_BY_KEY_INDEX_RANGE_REL);
+}
+
 bool opRequiresIndex(int op) {
 	return (op == OP_LIST_INSERT               || op == OP_LIST_INSERT_ITEMS  ||
 			op == OP_LIST_POP                  || op == OP_LIST_POP_RANGE     ||
@@ -176,7 +233,8 @@ bool opRequiresIndex(int op) {
 			op == OP_MAP_REMOVE_BY_INDEX       || op == OP_MAP_REMOVE_BY_RANK ||
 			op == OP_MAP_REMOVE_BY_RANK_RANGE  || op == OP_MAP_GET_BY_INDEX   ||
 			op == OP_MAP_GET_BY_INDEX_RANGE    || op == OP_MAP_GET_BY_RANK    ||
-			op == OP_MAP_GET_BY_RANK_RANGE);
+			op == OP_MAP_GET_BY_RANK_RANGE     || op == OP_MAP_REMOVE_BY_INDEX_RANGE ||
+			op == OP_LIST_INCREMENT);
 }
 
 bool opRequiresValue(int op) {
@@ -187,11 +245,13 @@ bool opRequiresValue(int op) {
 			op != OP_MAP_SET_POLICY      && op != OP_MAP_SIZE           &&
 			op != OP_MAP_CLEAR           && op != OP_MAP_REMOVE_BY_KEY  &&
 			op != OP_MAP_REMOVE_BY_INDEX && op != OP_MAP_REMOVE_BY_RANK &&
-			op != OP_MAP_GET_BY_KEY      && op != OP_MAP_GET_BY_INDEX);
+			op != OP_MAP_GET_BY_KEY      && op != OP_MAP_GET_BY_INDEX   &&
+			op != OP_MAP_GET_BY_KEY_RANGE && op != OP_MAP_GET_BY_RANK);
 }
 
 bool opRequiresRange(int op) {
-	return (op == OP_MAP_REMOVE_BY_VALUE_RANGE  || op == OP_MAP_GET_BY_VALUE_RANGE);
+	return (op == OP_MAP_REMOVE_BY_VALUE_RANGE  || op == OP_MAP_GET_BY_VALUE_RANGE ||
+			op == OP_MAP_GET_BY_KEY_RANGE);
 }
 
 bool opReturnsResult(int op) {
@@ -203,7 +263,7 @@ bool opReturnsResult(int op) {
 			op == OP_LIST_INSERT    || op == OP_LIST_INSERT_ITEMS ||
 			op == OP_LIST_POP       || op == OP_LIST_POP_RANGE    ||
 			op == OP_LIST_SET       || op == OP_MAP_GET_BY_KEY    ||
-			op == OP_MAP_GET_BY_KEY_RANGE);
+			op == OP_MAP_GET_BY_KEY_RANGE || op == OP_LIST_INCREMENT);
 }
 
 bool opRequiresMapPolicy(int op) {
@@ -245,13 +305,30 @@ as_status add_op(AerospikeClient * self, as_error * err, PyObject * py_val, as_v
 	PyObject * py_map_policy = NULL;
 	PyObject * py_return_type = NULL;
 	Py_ssize_t pos = 0;
+
+	if (get_operation(err, py_val, &operation) != AEROSPIKE_OK) {
+		return err->code;
+	}
+
+	/* Handle the list operations with a helper in the cdt_list_operate.c file */
+	if (isListOp(operation)) {
+		return add_new_list_op(self, err, py_val, unicodeStrVector, static_pool,
+			ops, operation, ret_type, SERIALIZER_PYTHON); //This hardcoding matches current behavior
+
+	}
+
+	if (isNewMapOp(operation)) {
+		return add_new_map_op(self, err, py_val, unicodeStrVector, static_pool,
+			ops, operation, ret_type, SERIALIZER_PYTHON);
+	}
+
 	while (PyDict_Next(py_val, &pos, &key_op, &value)) {
 		if (!PyString_Check(key_op)) {
 			return as_error_update(err, AEROSPIKE_ERR_CLIENT, "An operation key must be a string.");
 		} else {
 			char * name = PyString_AsString(key_op);
-			if (!strcmp(name,"op") && (PyInt_Check(value) || PyLong_Check(value))) {
-				operation = PyInt_AsLong(value);
+			if (!strcmp(name,"op")) {
+				continue;
 			} else if (!strcmp(name, "bin")) {
 				py_bin = value;
 			} else if (!strcmp(name, "index")) {
@@ -266,6 +343,9 @@ as_status add_op(AerospikeClient * self, as_error * err, PyObject * py_val, as_v
 				py_map_policy = value;
 			} else if (!strcmp(name, "return_type")) {
 				py_return_type = value;
+			}
+			else if (strcmp(name, "inverted") == 0) {
+				continue;
 			} else {
 				return as_error_update(err, AEROSPIKE_ERR_PARAM,
 						"Operation can contain only op, bin, index, key, val, return_type and map_policy keys");
@@ -330,6 +410,17 @@ as_status add_op(AerospikeClient * self, as_error * err, PyObject * py_val, as_v
 		}
 		return_type = PyInt_AsLong(py_return_type);
 	}
+
+	/* Add the inverted flag to the return type if it's present */
+	if (invertIfSpecified(err, py_val, &return_type) != AEROSPIKE_OK) {
+		return err->code;
+	}
+
+	if (err->code != AEROSPIKE_OK) {
+		return err->code;
+	}
+
+
 	*ret_type = return_type;
 
 	if (py_index) {
@@ -445,67 +536,6 @@ as_status add_op(AerospikeClient * self, as_error * err, PyObject * py_val, as_v
 			as_operations_add_write(ops, bin, (as_bin_value *) put_val);
 			break;
 
-		//------- LIST OPERATIONS ---------
-		case OP_LIST_APPEND:
-			CONVERT_VAL_TO_AS_VAL();
-			as_operations_add_list_append(ops, bin, put_val);
-			break;
-		case OP_LIST_APPEND_ITEMS:
-			CONVERT_VAL_TO_AS_VAL();
-			as_operations_add_list_append_items(ops, bin, (as_list*)put_val);
-			break;
-		case OP_LIST_INSERT:
-			CONVERT_VAL_TO_AS_VAL();
-			as_operations_add_list_insert(ops, bin, index, put_val);
-			break;
-		case OP_LIST_INSERT_ITEMS:
-			CONVERT_VAL_TO_AS_VAL();
-			as_operations_add_list_insert_items(ops, bin, index, (as_list*)put_val);
-			break;
-		case OP_LIST_POP:
-			as_operations_add_list_pop(ops, bin, index);
-			break;
-		case OP_LIST_POP_RANGE:
-			if (py_value && (pyobject_to_index(self, err, py_value, &offset) != AEROSPIKE_OK)) {
-				return err->code;
-			}
-			as_operations_add_list_pop_range(ops, bin, index, offset);
-			break;
-		case OP_LIST_REMOVE:
-			as_operations_add_list_remove(ops, bin, index);
-			break;
-		case OP_LIST_REMOVE_RANGE:
-			if (py_value && (pyobject_to_index(self, err, py_value, &offset) != AEROSPIKE_OK)) {
-				return err->code;
-			}
-			as_operations_add_list_remove_range(ops, bin, index, offset);
-			break;
-		case OP_LIST_CLEAR:
-			as_operations_add_list_clear(ops, bin);
-			break;
-		case OP_LIST_SET:
-			CONVERT_VAL_TO_AS_VAL();
-			as_operations_add_list_set(ops, bin, index, put_val);
-			break;
-		case OP_LIST_GET:
-			as_operations_add_list_get(ops, bin, index);
-			break;
-		case OP_LIST_GET_RANGE:
-			if (py_value && (pyobject_to_index(self, err, py_value, &offset) != AEROSPIKE_OK)) {
-				return err->code;
-			}
-			as_operations_add_list_get_range(ops, bin, index, offset);
-			break;
-		case OP_LIST_TRIM:
-			if (py_value && (pyobject_to_index(self, err, py_value, &offset) != AEROSPIKE_OK)) {
-				return err->code;
-			}
-			as_operations_add_list_trim(ops, bin, index, offset);
-			break;
-		case OP_LIST_SIZE:
-			as_operations_add_list_size(ops, bin);
-			break;
-
 		//------- MAP OPERATIONS ---------
 		case OP_MAP_SET_POLICY:
 			as_operations_add_map_set_policy(ops, bin, &map_policy);
@@ -588,6 +618,10 @@ as_status add_op(AerospikeClient * self, as_error * err, PyObject * py_val, as_v
 			CONVERT_KEY_TO_AS_VAL();
 			as_operations_add_map_get_by_key_range(ops, bin, put_key, put_range, return_type);
 			break;
+		case OP_MAP_GET_BY_KEY_LIST:
+			CONVERT_VAL_TO_AS_VAL();
+			as_operations_add_map_get_by_key_list(ops, bin, (as_list *)put_val, return_type);
+			break;
 		case OP_MAP_GET_BY_VALUE:
 			CONVERT_VAL_TO_AS_VAL();
 			as_operations_add_map_get_by_value(ops, bin, put_val, return_type);
@@ -597,6 +631,10 @@ as_status add_op(AerospikeClient * self, as_error * err, PyObject * py_val, as_v
 			CONVERT_RANGE_TO_AS_VAL();
 			as_operations_add_map_get_by_value_range(ops, bin, put_val, put_range, return_type);
 			break;
+		case OP_MAP_GET_BY_VALUE_LIST:
+			CONVERT_VAL_TO_AS_VAL();
+			as_operations_add_map_get_by_value_list(ops, bin, (as_list *)put_val, return_type);
+			break;
 		case OP_MAP_GET_BY_INDEX:
 			as_operations_add_map_get_by_index(ops, bin, index, return_type);
 			break;
@@ -703,11 +741,7 @@ PyObject *  AerospikeClient_Operate_Invoke(
 		goto CLEANUP;
 	}
 	if (rec) {
-		if (return_type == AS_MAP_RETURN_KEY_VALUE) {
-			record_to_pyobject_cnvt_list_to_map(self, err, rec, key, &py_rec);
-		} else {
-			record_to_pyobject(self, err, rec, key, &py_rec);
-		}
+		record_to_pyobject(self, err, rec, key, &py_rec);
 	}
 
 CLEANUP:
@@ -808,139 +842,132 @@ static PyObject *  AerospikeClient_OperateOrdered_Invoke(
 	PyObject * py_policy)
 {
 	long operation;
-	long return_type;
-	int i = 0;
+	long return_type = -1;
 	PyObject * py_rec = NULL;
+	as_record * rec = NULL;
 	as_policy_operate operate_policy;
 	as_policy_operate *operate_policy_p = NULL;
 
 	as_vector * unicodeStrVector = as_vector_create(sizeof(char *), 128);
 
-	if (py_policy) {
-		if(pyobject_to_policy_operate(err, py_policy, &operate_policy, &operate_policy_p,
-				&self->as->config.policies.operate) != AEROSPIKE_OK) {
-			goto CLEANUP;
-		}
-	}
-
 	as_static_pool static_pool;
 	memset(&static_pool, 0, sizeof(static_pool));
 
-	Py_ssize_t size = PyList_Size(py_list);
+	as_operations ops;
+	Py_ssize_t ops_list_size = PyList_Size(py_list);
+	as_operations_inita(&ops, ops_list_size);
 
-	if (!self || !self->as) {
-		as_error_update(err, AEROSPIKE_ERR_PARAM, "Invalid aerospike object");
-		goto CLEANUP;
+	/* These are the values which will be returned in a 3 element list */
+	PyObject* py_return_key = NULL;
+	PyObject* py_return_meta = NULL;
+	PyObject* py_return_bins = NULL;
+
+	CHECK_CONNECTED(err);
+
+	if (py_policy) {
+		if (pyobject_to_policy_operate(err, py_policy, &operate_policy,
+				&operate_policy_p, &self->as->config.policies.operate) != AEROSPIKE_OK) {
+			goto CLEANUP;
+		}
+	}
+
+	if (py_meta) {
+		if (check_for_meta(py_meta, &ops, err) != AEROSPIKE_OK) {
+			goto CLEANUP;
+		}
+	}
+
+	for (Py_ssize_t i = 0; i < ops_list_size; i++) {
+
+		PyObject* py_current_op = NULL;
+		py_current_op = PyList_GetItem(py_list, i);
+
+		if (PyDict_Check(py_current_op)) {
+			if (add_op(self, err, py_current_op, unicodeStrVector, &static_pool,
+					&ops, &operation, &return_type) != AEROSPIKE_OK) {
+				goto CLEANUP;
+			}
+		} else {
+			as_error_update(err, AEROSPIKE_ERR_PARAM, "Operation must be a dict");
+			goto CLEANUP;
+		}
 	}
 
 	if (err->code != AEROSPIKE_OK) {
+		as_error_update(err, err->code, NULL);
 		goto CLEANUP;
 	}
 
-	PyObject * py_rec_key = NULL;
-	PyObject * py_rec_meta = NULL;
-	PyObject * py_bins = NULL;
+	// Initialize record
+	as_record_init(rec, 0);
 
-	py_bins = PyList_New(0);
+	Py_BEGIN_ALLOW_THREADS
+	aerospike_key_operate(self->as, err, operate_policy_p, key, &ops, &rec);
+	Py_END_ALLOW_THREADS
 
-	for (i = 0; i < size; i++) {
-		as_operations ops;
-		as_operations_init(&ops, 1);
-
-		as_record * rec = NULL;
-
-		if (py_meta) {
-			if (check_for_meta(py_meta, &ops, err) != AEROSPIKE_OK) {
-				goto LOOP_CLEANUP;
-			}
-		}
-
-		PyObject * py_val = PyList_GetItem(py_list, i);
-		operation = -1;
-		return_type = -1;
-		if (PyDict_Check(py_val)) {
-			if (add_op(self, err, py_val, unicodeStrVector, &static_pool, &ops, &operation, &return_type) != AEROSPIKE_OK) {
-				goto LOOP_CLEANUP;
-			}
-		}
-
-		Py_BEGIN_ALLOW_THREADS
-		aerospike_key_operate(self->as, err, operate_policy_p, key, &ops, &rec);
-		Py_END_ALLOW_THREADS
-
-		if (err->code != AEROSPIKE_OK) {
-			as_error_update(err, err->code, NULL);
-			goto LOOP_CLEANUP;
-		}
-
-		if (rec) {
-			PyObject *py_rec_bins = NULL;
-			if (i == 0) {
-				key_to_pyobject(err, key ? key : &rec->key, &py_rec_key);
-				metadata_to_pyobject(err, rec, &py_rec_meta);
-			}
-			bins_to_pyobject(self, err, rec, &py_rec_bins, return_type == AS_MAP_RETURN_KEY_VALUE);
-
-			if (opReturnsResult(operation))
-			{
-				PyObject *py_value = PyDict_GetItemString(py_rec_bins, ops.binops.entries->bin.name);
-				PyObject *py_rec_tuple = PyTuple_New(2);
-				if (py_value) {
-					Py_INCREF(py_value);
-					PyTuple_SetItem(py_rec_tuple, 0, PyString_FromString(ops.binops.entries->bin.name));
-					PyTuple_SetItem(py_rec_tuple, 1, py_value);
-				} else {
-					Py_INCREF(Py_None);
-					PyTuple_SetItem(py_rec_tuple, 0, PyString_FromString(ops.binops.entries->bin.name));
-					PyTuple_SetItem(py_rec_tuple, 1, Py_None);
-				}
-
-				PyList_Append(py_bins, py_rec_tuple);
-				Py_DECREF(py_rec_tuple);
-			} else {
-				Py_INCREF(Py_None);
-				PyList_Append(py_bins, Py_None);
-			}
-			Py_DECREF(py_rec_bins);
-		}
-
-LOOP_CLEANUP:
-		as_operations_destroy(&ops);
-
-		if (rec) {
-			as_record_destroy(rec);
-		}
-
-		if (err->code != AEROSPIKE_OK && i == 0) {
-			goto CLEANUP;
-		} else if (err->code != AEROSPIKE_OK) {
-			as_error_reset(err);
-			break;
-		}
+	if (err->code != AEROSPIKE_OK) {
+		as_error_update(err, err->code, NULL);
+		goto CLEANUP;
 	}
 
-	py_rec = PyTuple_New(3);
+	if (rec) {
+		/* Build the return tuple: (key, meta, bins) */
+		key_to_pyobject(err, key, &py_return_key);
+		if (err->code != AEROSPIKE_OK || !py_return_key) {
+			goto CLEANUP;
+		}
 
-	PyTuple_SetItem(py_rec, 0, py_rec_key);
-	PyTuple_SetItem(py_rec, 1, py_rec_meta);
-	PyTuple_SetItem(py_rec, 2, py_bins);
+		metadata_to_pyobject(err, rec, &py_return_meta);
+		if (err->code != AEROSPIKE_OK || !py_return_meta) {
+			Py_XDECREF(py_return_key);
+			goto CLEANUP;
+		}
+
+		operate_bins_to_pyobject(self, err, rec, &py_return_bins);
+		if (err->code != AEROSPIKE_OK || !py_return_bins) {
+			Py_XDECREF(py_return_key);
+			Py_XDECREF(py_return_meta);
+			goto CLEANUP;
+		}
+
+
+		py_rec = Py_BuildValue("OOO", py_return_key, py_return_meta, py_return_bins);
+		if (!py_rec) {
+			as_error_update(err, AEROSPIKE_ERR_CLIENT, "Unable to build return tuple");
+		}
+
+		/* If Py_BuildValue succeeded it increased the reference count of all 3 of these to 2,
+		 * so we decref them.*
+		 * If Py_BuildValue failed, we aren't returning anything, so they need to be
+		 * decref'd in that case as well.
+		 */
+		Py_XDECREF(py_return_key);
+		Py_XDECREF(py_return_bins);
+		Py_XDECREF(py_return_meta);
+	}
 
 CLEANUP:
 	for (unsigned int i=0; i<unicodeStrVector->size ; i++) {
 		free(as_vector_get_ptr(unicodeStrVector, i));
 	}
+
 	as_vector_destroy(unicodeStrVector);
 
+	if (rec) {
+		as_record_destroy(rec);
+	}
 	if (key->valuep) {
 		as_key_destroy(key);
 	}
 
+	as_operations_destroy(&ops);
+
 	if (err->code != AEROSPIKE_OK) {
 		PyObject * py_err = NULL;
 		error_to_pyobject(err, &py_err);
 		PyObject *exception_type = raise_exception(err);
 		PyErr_SetObject(exception_type, py_err);
-		Py_DECREF(py_err);
+		Py_XDECREF(py_err);
 		return NULL;
 	}
 
@@ -1199,3 +1226,46 @@ CLEANUP:
 	return PyLong_FromLong(0);
 }
 
+static as_status
+get_operation(as_error* err, PyObject* op_dict, long* operation_ptr)
+{
+        PyObject* py_operation = PyDict_GetItemString(op_dict, PY_OPERATION_KEY);
+        if (!py_operation) {
+            return as_error_update(err, AEROSPIKE_ERR_PARAM, "Operation must contain an \"op\" entry");
+        }
+        if (!PyInt_Check(py_operation)) {
+            return as_error_update(err, AEROSPIKE_ERR_PARAM, "Operation must be an integer");
+        }
+
+        *operation_ptr = PyLong_AsLong(py_operation);
+        if (PyErr_Occurred()) {
+            if (*operation_ptr == -1 && PyErr_ExceptionMatches(PyExc_OverflowError)) {
+                return as_error_update(err, AEROSPIKE_ERR_PARAM, "Operation code too large");
+            }
+            else {
+                return as_error_update(err, AEROSPIKE_ERR_PARAM, "Invalid operation");
+            }
+        }
+        return AEROSPIKE_OK;
+}
+
+static as_status
+invertIfSpecified(as_error* err, PyObject* op_dict, uint64_t* return_value) {
+	PyObject* pyInverted = PyDict_GetItemString(op_dict, "inverted");
+	int truthValue;
+	if (!pyInverted) {
+		return AEROSPIKE_OK;
+	}
+	truthValue = PyObject_IsTrue(pyInverted);
+
+	/* An error ocurred, update the flag */
+	if (truthValue == -1) {
+		return as_error_update(err, AEROSPIKE_ERR_PARAM, "Invalid inverted value");
+	}
+
+	if (truthValue) {
+		*return_value |= AS_MAP_RETURN_INVERTED;
+	}
+
+	return AEROSPIKE_OK;
+}
\ No newline at end of file
