@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2013-2016 Aerospike, Inc.
+ * Copyright 2013-2017 Aerospike, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -25,7 +25,6 @@
 #include <aerospike/as_record.h>
 #include <aerospike/as_geojson.h>
 
-#include <aerospike/as_ldt.h>
 #include <aerospike/as_arraylist.h>
 #include <aerospike/as_hashmap.h>
 #include <aerospike/as_list.h>
@@ -35,6 +34,7 @@
 #include <aerospike/as_operations.h>
 #include <aerospike/as_bytes.h>
 #include <aerospike/as_double.h>
+#include <aerospike/as_record_iterator.h>
 
 #include "conversions.h"
 #include "geo.h"
@@ -51,6 +51,7 @@
 #define PY_EXCEPTION_MSG 1
 #define PY_EXCEPTION_FILE 2
 #define PY_EXCEPTION_LINE 3
+#define AS_PY_EXCEPTION_IN_DOUBT 4
 
 as_status as_udf_file_to_pyobject( as_error *err, as_udf_file * entry, PyObject ** py_file )
 {
@@ -272,22 +273,54 @@ as_status as_privilege_to_pyobject( as_error * err, as_privilege privileges[], P
 	return err->code;
 }
 
-as_status pyobject_to_strArray( as_error * err, PyObject * py_list,  char ** arr )
+as_status pyobject_to_strArray( as_error * err, PyObject * py_list,  char ** arr , uint32_t max_len)
 {
-	as_error_reset(err);
 
-	Py_ssize_t size = PyList_Size(py_list);
+	as_error_reset(err);
+	PyObject* py_u_str = NULL;
+
 	if (!PyList_Check(py_list)) {
 		return as_error_update(err, AEROSPIKE_ERR_CLIENT, "not a list");
 	}
 
+	Py_ssize_t size = PyList_Size(py_list);
+
 	char *s;
 	for ( int i = 0; i < size; i++ ) {
 		PyObject * py_val = PyList_GetItem(py_list, i);
+
 		if (PyString_Check(py_val)) {
 			s = PyString_AsString(py_val);
-			strcpy(arr[i], s);
+
+			if (strlen(s) < max_len) {
+				strcpy(arr[i], s);
+			} else {
+				as_error_update(err, AEROSPIKE_ERR_CLIENT, "String exceeds max length");
+				return err->code;
+			}
+
+		} else if (PyUnicode_Check(py_val)) {
+			py_u_str = PyUnicode_AsUTF8String(py_val);
+			if (!py_u_str) {
+				as_error_update(err, AEROSPIKE_ERR_CLIENT, "Unable to convert unicode string");
+				return err->code;
+			}
+			s = PyBytes_AsString(py_u_str);
+
+			if (strlen(s) < max_len) {
+				strcpy(arr[i], s);
+				Py_DECREF(py_u_str);
+			} else {
+				Py_DECREF(py_u_str);
+				as_error_update(err, AEROSPIKE_ERR_CLIENT, "String exceeds max length");
+				return err->code;
+			}
+
+		} else {
+			as_error_update(err, AEROSPIKE_ERR_CLIENT, "Item is not a string");
+			return err->code;
 		}
+
 	}
 
 	return err->code;
@@ -640,51 +673,56 @@ as_status pyobject_to_record(AerospikeClient * self, as_error * err, PyObject *
 			}
 		}
 
-		if (py_meta && PyDict_Check(py_meta)) {
-			PyObject * py_gen = PyDict_GetItemString(py_meta, "gen");
-			PyObject * py_ttl = PyDict_GetItemString(py_meta, "ttl");
+		if (py_meta  && py_meta != Py_None) {
+			if (!PyDict_Check(py_meta)) {
+				as_error_update(err, AEROSPIKE_ERR_PARAM, "meta must be a dictionary");
+			} else {
+				PyObject * py_gen = PyDict_GetItemString(py_meta, "gen");
+				PyObject * py_ttl = PyDict_GetItemString(py_meta, "ttl");
 
-			if (py_ttl) {
-				if (PyInt_Check(py_ttl)) {
-					rec->ttl = (uint32_t) PyInt_AsLong(py_ttl);
-					if (rec->ttl == (uint32_t)-1 && PyErr_Occurred()) {
-						if (PyErr_ExceptionMatches(PyExc_OverflowError)) {
-							as_error_update(err, AEROSPIKE_ERR_PARAM, "integer value exceeds sys.maxsize");
+				if (py_ttl) {
+					if (PyInt_Check(py_ttl)) {
+						rec->ttl = (uint32_t) PyInt_AsLong(py_ttl);
+						if (rec->ttl == (uint32_t)-1 && PyErr_Occurred()) {
+							if (PyErr_ExceptionMatches(PyExc_OverflowError)) {
+								as_error_update(err, AEROSPIKE_ERR_PARAM, "integer value exceeds sys.maxsize");
+							}
 						}
-					}
-				} else if (PyLong_Check(py_ttl)) {
-					rec->ttl = (uint32_t) PyLong_AsLongLong(py_ttl);
-					if (rec->ttl == (uint32_t)-1 && PyErr_Occurred()) {
-						if (PyErr_ExceptionMatches(PyExc_OverflowError)) {
-							as_error_update(err, AEROSPIKE_ERR_PARAM, "integer value exceeds sys.maxsize");
+					} else if (PyLong_Check(py_ttl)) {
+						rec->ttl = (uint32_t) PyLong_AsLongLong(py_ttl);
+						if (rec->ttl == (uint32_t)-1 && PyErr_Occurred()) {
+							if (PyErr_ExceptionMatches(PyExc_OverflowError)) {
+								as_error_update(err, AEROSPIKE_ERR_PARAM, "integer value exceeds sys.maxsize");
+							}
 						}
+					} else {
+						as_error_update(err, AEROSPIKE_ERR_PARAM, "TTL should be an int or long");
 					}
-				} else {
-					as_error_update(err, AEROSPIKE_ERR_PARAM, "TTL should be an int or long");
 				}
-			}
 
-			if (py_gen) {
-				if (PyInt_Check(py_gen)) {
-					rec->gen = (uint16_t) PyInt_AsLong(py_gen);
-					if (rec->gen == (uint16_t)-1 && PyErr_Occurred()) {
-						if (PyErr_ExceptionMatches(PyExc_OverflowError)) {
-							as_error_update(err, AEROSPIKE_ERR_PARAM, "integer value exceeds sys.maxsize");
+				if (py_gen) {
+					if (PyInt_Check(py_gen)) {
+						rec->gen = (uint16_t) PyInt_AsLong(py_gen);
+						if (rec->gen == (uint16_t)-1 && PyErr_Occurred()) {
+							if (PyErr_ExceptionMatches(PyExc_OverflowError)) {
+								as_error_update(err, AEROSPIKE_ERR_PARAM, "integer value exceeds sys.maxsize");
+							}
 						}
-					}
-				} else if (PyLong_Check(py_gen)) {
-					rec->gen = (uint16_t) PyLong_AsLongLong(py_gen);
-					if (rec->gen == (uint16_t)-1 && PyErr_Occurred()) {
-						if (PyErr_ExceptionMatches(PyExc_OverflowError)) {
-							as_error_update(err, AEROSPIKE_ERR_PARAM, "integer value exceeds sys.maxsize");
+					} else if (PyLong_Check(py_gen)) {
+						rec->gen = (uint16_t) PyLong_AsLongLong(py_gen);
+						if (rec->gen == (uint16_t)-1 && PyErr_Occurred()) {
+							if (PyErr_ExceptionMatches(PyExc_OverflowError)) {
+								as_error_update(err, AEROSPIKE_ERR_PARAM, "integer value exceeds sys.maxsize");
+							}
 						}
+					} else {
+						as_error_update(err, AEROSPIKE_ERR_PARAM, "Generation should be an int or long");
 					}
-				} else {
-					as_error_update(err, AEROSPIKE_ERR_PARAM, "Generation should be an int or long");
 				}
 			}
 		}
 
+
 		if (err->code != AEROSPIKE_OK) {
 			as_record_destroy(rec);
 		}
@@ -1160,11 +1198,23 @@ static bool map_to_pyobject_each(const as_val * key, const as_val * val, void *
 	val_to_pyobject(convd->client, convd->err, val, &py_val);
 
 	if (err->code != AEROSPIKE_OK) {
-		PyObject_Del(py_key);
+		Py_DECREF(py_key);
 		return false;
 	}
 
-	PyDict_SetItem(py_dict, py_key, py_val);
+	/* We failed to set a dictionary item. This is probably
+	 * due to an unhashable keytype
+	 */
+	if(PyDict_SetItem(py_dict, py_key, py_val) == -1) {
+		Py_CLEAR(py_key);
+		Py_CLEAR(py_val);
+		if (PyErr_Occurred() && PyErr_ExceptionMatches(PyExc_TypeError)) {
+			as_error_update(err, AEROSPIKE_ERR_CLIENT, "Unable to use unhashable type as a dictionary key");
+		} else {
+			as_error_update(err, AEROSPIKE_ERR_CLIENT, "Unable to add dictionary item");
+		}
+		return false;
+	}
 
 	Py_DECREF(py_key);
 	Py_DECREF(py_val);
@@ -1187,7 +1237,7 @@ as_status map_to_pyobject(AerospikeClient * self, as_error * err, const as_map *
 	as_map_foreach(map, map_to_pyobject_each, &convd);
 
 	if (err->code != AEROSPIKE_OK) {
-		PyObject_Del(*py_map);
+		Py_DECREF(*py_map);
 		return err->code;
 	}
 
@@ -1197,6 +1247,7 @@ as_status map_to_pyobject(AerospikeClient * self, as_error * err, const as_map *
 as_status do_record_to_pyobject(AerospikeClient * self, as_error * err, const as_record * rec, const as_key * key, PyObject ** obj, bool cnvt_list_to_map)
 {
 	as_error_reset(err);
+	*obj = NULL;
 
 	if (!rec) {
 		return as_error_update(err, AEROSPIKE_ERR_CLIENT, "record is null");
@@ -1207,9 +1258,21 @@ as_status do_record_to_pyobject(AerospikeClient * self, as_error * err, const as
 	PyObject * py_rec_meta = NULL;
 	PyObject * py_rec_bins = NULL;
 
-	key_to_pyobject(err, key ? key : &rec->key, &py_rec_key);
-	metadata_to_pyobject(err, rec, &py_rec_meta);
-	bins_to_pyobject(self, err, rec, &py_rec_bins, cnvt_list_to_map);
+
+	if (key_to_pyobject(err, key ? key : &rec->key, &py_rec_key) != AEROSPIKE_OK) {
+		return err->code;
+	}
+
+	if (metadata_to_pyobject(err, rec, &py_rec_meta) != AEROSPIKE_OK) {
+		Py_CLEAR(py_rec_key);
+		return err->code;
+	}
+
+	if (bins_to_pyobject(self, err, rec, &py_rec_bins, cnvt_list_to_map) != AEROSPIKE_OK) {
+		Py_CLEAR(py_rec_key);
+		Py_CLEAR(py_rec_meta);
+		return err->code;
+	}
 
 	if (!py_rec_key) {
 		Py_INCREF(Py_None);
@@ -1410,6 +1473,66 @@ as_status bins_to_pyobject(AerospikeClient * self, as_error * err, const as_reco
 	return err->code;
 }
 
+/*
+ * operate_bins_to_pyobject
+ *
+ * Because operate returns a record with multiple entries per bin value, this function
+ * iterates over each bin and adds a tuple of the form (bin_name, bin_value). These tuples
+ * are stored in a list.
+ * e.g. [('bin1', 5), ('bin1', 6), ('bin2', [3,4,5])]
+ * Other converter functions assume one entry per bin name.
+ * @param self A pointer to a Python Client aerospike wrapper, needed for deserialization
+ * @param err pointer to as_error. It will be filled if an error is encountered.
+ * @param rec the C client record to be converted to a list of tuples of bin values
+ * @param py_bins Pointer to pointer to PyObject. It will be allocated and filled with list of tuples of bin values
+ */
+as_status operate_bins_to_pyobject(AerospikeClient * self, as_error * err, const as_record * rec, PyObject ** py_bins)
+{
+	as_error_reset(err);
+	PyObject* py_bin_value = NULL;
+	PyObject* py_bin_pair = NULL;
+
+	if (!rec) {
+		// this should never happen, but if it did...
+		return as_error_update(err, AEROSPIKE_ERR_CLIENT, "record is null");
+	}
+
+	*py_bins = PyList_New(0);
+	as_record_iterator it;
+	as_record_iterator_init(&it, rec);
+
+	while ( as_record_iterator_has_next(&it) ) {
+	    as_bin * bin = as_record_iterator_next(&it);
+	    py_bin_value = NULL;
+	    py_bin_pair = NULL;
+	    val_to_pyobject(self, err, (as_val*)as_bin_get_value(bin), &py_bin_value);
+	    if (err->code != AEROSPIKE_OK) {
+	    	goto CLEANUP;
+	    }
+	    if (!py_bin_value) {
+	    	as_error_update(err, AEROSPIKE_ERR_CLIENT, "Null entry in operate ordered conversion");
+	    	goto CLEANUP;
+	    }
+	    py_bin_pair = Py_BuildValue("sO", as_bin_get_name(bin), py_bin_value);
+	    if (!py_bin_pair) {
+	    	as_error_update(err, AEROSPIKE_ERR_CLIENT, "Unable to build bin entry");
+		    Py_DECREF(py_bin_value);
+	    	goto CLEANUP;
+	    }
+	    Py_DECREF(py_bin_value);
+	    PyList_Append(*py_bins, py_bin_pair);
+	    Py_DECREF(py_bin_pair);
+	}
+
+CLEANUP:
+	if (err->code != AEROSPIKE_OK) {
+		Py_DECREF(*py_bins);
+	}
+
+	as_record_iterator_destroy(&it);
+	return err->code;
+}
+
 as_status metadata_to_pyobject(as_error * err, const as_record * rec, PyObject ** obj)
 {
 	as_error_reset(err);
@@ -1455,39 +1578,19 @@ bool error_to_pyobject(const as_error * err, PyObject ** obj)
 	PyObject * py_code = PyLong_FromLongLong(err->code);
 	PyObject * py_message = PyString_FromString(err->message);
 
-	PyObject * py_err = PyTuple_New(4);
+	PyObject* py_in_doubt = err->in_doubt ? Py_True : Py_False;
+	Py_INCREF(py_in_doubt);
+
+	PyObject * py_err = PyTuple_New(5);
 	PyTuple_SetItem(py_err, PY_EXCEPTION_CODE, py_code);
 	PyTuple_SetItem(py_err, PY_EXCEPTION_MSG, py_message);
 	PyTuple_SetItem(py_err, PY_EXCEPTION_FILE, py_file);
 	PyTuple_SetItem(py_err, PY_EXCEPTION_LINE, py_line);
+	PyTuple_SetItem(py_err, AS_PY_EXCEPTION_IN_DOUBT, py_in_doubt);
 	*obj = py_err;
 	return true;
 }
 
-/**
- * This method will initialize ldt.
- *
- * @param error                 The error parameter
- * @param ldt_p                 The LDT instance
- * @param bin_name              The ldt bin name
- * @param type                  The type of LDT
- * @param module                The UDF module
- *
- * On failure it will set an error.
- */
-as_status initialize_ldt(as_error *error, as_ldt* ldt_p, char* bin_name,
-		int type, char* module)
-{
-	as_error_reset(error);
-	if (!bin_name) {
-		as_error_update(error, AEROSPIKE_ERR_PARAM, "Bin name is null");
-	}
-	if (!as_ldt_init(ldt_p, bin_name, type, module)) {
-		as_error_update(error, AEROSPIKE_ERR_PARAM, "Unable to initialize LDT");
-	}
-
-	return error->code;
-}
 
 void initialize_bin_for_strictypes(AerospikeClient *self, as_error *err, PyObject *py_value, as_binop *binop, char *bin, as_static_pool *static_pool) {
 
@@ -1651,7 +1754,7 @@ as_status check_for_meta(PyObject * py_meta, as_operations * ops, as_error *err)
 			}
 			ops->gen = gen;
 		}
-	} else {
+	} else if (py_meta && (py_meta != Py_None)) {
 		return as_error_update(err, AEROSPIKE_ERR_PARAM, "Metadata should be of type dictionary");
 	}
 	return err->code;
@@ -1675,3 +1778,131 @@ as_status pyobject_to_index(AerospikeClient * self, as_error * err, PyObject * p
 	return err->code;
 }
 
+as_status
+as_batch_read_results_to_pyobject(as_error* err, AerospikeClient* client, const as_batch_read* results,
+		uint32_t size, PyObject** py_records)
+{
+	*py_records = NULL;
+	PyObject* temp_py_recs = PyList_New(0);
+
+	if (!temp_py_recs) {
+		return as_error_update(err, AEROSPIKE_ERR_CLIENT, "Failed to allocate memory for batch results");
+	}
+
+	// Loop over results array
+	for (uint32_t i = 0; i < size; i++) {
+		PyObject * py_rec = NULL;
+		PyObject * py_key = NULL;
+		if (results[i].result == AEROSPIKE_OK) {
+			/* There was a record for the item, but we failed to convert it, probably a deserialize issue, error out */
+			record_to_pyobject(client, err, &results[i].record, results[i].key, &py_rec);
+			if (!py_rec || err->code != AEROSPIKE_OK) {
+				Py_XDECREF(temp_py_recs);
+				return err->code;
+			}
+		/* The record wasn't found, build a (key, None, None) tuple */
+		} else {
+			key_to_pyobject(err, results[i].key, &py_key);
+			if (!py_key || err->code != AEROSPIKE_OK) {
+				Py_XDECREF(temp_py_recs);
+				return err->code;
+			}
+			py_rec = Py_BuildValue("OOO", py_key, Py_None, Py_None);
+			Py_DECREF(py_key);
+		}
+
+		if (!py_rec) {
+			/* This means that build value, failed, so we are in trouble*/
+			Py_XDECREF(temp_py_recs);
+			return as_error_update(err, AEROSPIKE_ERR_CLIENT, "Failed to allocate memory for record entry");
+		}
+
+		if (PyList_Append(temp_py_recs, py_rec) != 0) {
+			Py_DECREF(py_rec);
+			Py_DECREF(temp_py_recs);
+			return as_error_update(err, AEROSPIKE_ERR_CLIENT, "Failed to add record to results");
+		}
+		Py_DECREF(py_rec);
+
+	}
+
+	// Release Python State
+	*py_records = temp_py_recs;
+	return AEROSPIKE_OK;
+}
+
+as_status
+batch_read_records_to_pyobject(AerospikeClient *self, as_error *err, as_batch_read_records* records, PyObject **py_recs)
+{
+	*py_recs = PyList_New(0);
+
+	if (!(*py_recs)) {
+		return as_error_update(err, AEROSPIKE_ERR_CLIENT, "Failed to allocate return list of records");
+	}
+	as_vector* list = &records->list;
+	for (uint32_t i = 0; i < list->size; i++) {
+
+		as_batch_read_record* batch = as_vector_get(list, i);
+		PyObject* py_rec = NULL;
+		PyObject* py_key = NULL;
+
+		/* There should be a record, so convert it to a tuple */
+		if (batch->result == AEROSPIKE_OK) {
+			record_to_pyobject(self, err, &batch->record, &batch->key, &py_rec);
+			if (!py_rec || err->code != AEROSPIKE_OK) {
+				Py_CLEAR(*py_recs);
+				return err->code;
+			}
+		/* No record, convert to (key, None, None) */
+		} else {
+			key_to_pyobject(err, &batch->key, &py_key);
+			if (!py_key || err->code != AEROSPIKE_OK) {
+				Py_CLEAR(*py_recs);
+				return err->code;
+			}
+			py_rec = Py_BuildValue("OOO", py_key, Py_None, Py_None);
+			Py_DECREF(py_key);
+			if (!py_rec) {
+				as_error_update(err, AEROSPIKE_ERR_CLIENT, "Failed to create a record tuple");
+				Py_CLEAR(*py_recs);
+				return err->code;
+			}
+		}
+
+		if (PyList_Append(*py_recs, py_rec) != 0) {
+			as_error_update(err, AEROSPIKE_ERR_CLIENT, "Failed to add record tuple to return list");
+			Py_XDECREF(py_rec);
+			Py_CLEAR(*py_recs);
+			return err->code;
+		}
+		Py_DECREF(py_rec);
+	}
+	return AEROSPIKE_OK;
+}
+
+/*
+This fetches a string from a Python String like. If it is a unicode in Python27, we need to convert it
+to a bytes like object first, and keep track of the intermediate object for later deletion.
+*/
+as_status
+string_and_pyuni_from_pystring(PyObject* py_string, PyObject** pyuni_r, char** c_str_ptr, as_error* err) {
+	/* Not needed if we drop support for Python > 3 < 3.3 */
+
+	PyObject* intermediate_uni = NULL;
+	*c_str_ptr = NULL;
+	if (PyString_Check(py_string)) {
+		*c_str_ptr = PyString_AsString(py_string);
+		return AEROSPIKE_OK;
+
+	} else if (PyUnicode_Check(py_string)) {
+		intermediate_uni = PyUnicode_AsUTF8String(py_string);
+		if (!intermediate_uni) {
+			return as_error_update(err, AEROSPIKE_ERR_PARAM, "Invalid unicode value");
+		}
+
+		*c_str_ptr = PyBytes_AsString(intermediate_uni);
+		*pyuni_r = intermediate_uni;
+		return AEROSPIKE_OK;
+	}
+	return as_error_update(err, AEROSPIKE_ERR_PARAM, "String value required");
+}
\ No newline at end of file
