@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2013-2016 Aerospike, Inc.
+ * Copyright 2013-2017 Aerospike, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -23,7 +23,6 @@
 #include <aerospike/as_status.h>
 #include <aerospike/as_admin.h>
 #include <aerospike/as_operations.h>
-#include <aerospike/as_map_operations.h>
 #include <aerospike/aerospike_index.h>
 #include "aerospike/as_scan.h"
 #include "aerospike/as_job.h"
@@ -56,6 +55,31 @@ __policy##_init(policy);\
 	}\
 }
 
+#define POLICY_SET_BASE_FIELD(__field, __type) { \
+	PyObject * py_field = PyDict_GetItemString(py_policy, #__field);\
+	if (py_field) {\
+		if (PyInt_Check(py_field)) {\
+			policy->base.__field = (__type) PyInt_AsLong(py_field);\
+		}\
+		else {\
+			return as_error_update(err, AEROSPIKE_ERR_PARAM, "%s is invalid", #__field);\
+		}\
+	}\
+}
+
+/* This allows the old timeout field to properly set timeout, remove in future release: > 3.1 */
+#define POLICY_SET_TOTAL_TIMEOUT_FROM_TIMEOUT() { \
+	PyObject * py_field = PyDict_GetItemString(py_policy, "timeout");\
+	if (py_field) {\
+		if (PyInt_Check(py_field)) {\
+			policy->base.total_timeout = (uint32_t) PyInt_AsLong(py_field);\
+		}\
+		else {\
+			return as_error_update(err, AEROSPIKE_ERR_PARAM, "timeout is invalid");\
+		}\
+	}\
+}
+
 #define MAP_POLICY_SET_FIELD(__field) { \
 	PyObject * py_field = PyDict_GetItemString(py_policy, #__field);\
 	if (py_field) {\
@@ -137,6 +161,7 @@ AerospikeConstants aerospike_constants[] = {
 	{ OP_LIST_GET_RANGE                     ,   "OP_LIST_GET_RANGE"},
 	{ OP_LIST_TRIM                          ,   "OP_LIST_TRIM"},
 	{ OP_LIST_SIZE                          ,   "OP_LIST_SIZE"},
+	{ OP_LIST_INCREMENT                     ,   "OP_LIST_INCREMENT"},
 
 	{ OP_MAP_SET_POLICY                     ,   "OP_MAP_SET_POLICY"},
 	{ OP_MAP_PUT                            ,   "OP_MAP_PUT"},
@@ -163,6 +188,8 @@ AerospikeConstants aerospike_constants[] = {
 	{ OP_MAP_GET_BY_INDEX_RANGE             ,   "OP_MAP_GET_BY_INDEX_RANGE"},
 	{ OP_MAP_GET_BY_RANK                    ,   "OP_MAP_GET_BY_RANK"},
 	{ OP_MAP_GET_BY_RANK_RANGE              ,   "OP_MAP_GET_BY_RANK_RANGE"},
+	{ OP_MAP_GET_BY_VALUE_LIST, "OP_MAP_GET_BY_VALUE_LIST"},
+	{ OP_MAP_GET_BY_KEY_LIST, "OP_MAP_GET_BY_KEY_LIST" },
 
 	{ AS_MAP_UNORDERED                      ,   "MAP_UNORDERED"},
 	{ AS_MAP_KEY_ORDERED                    ,   "MAP_KEY_ORDERED"},
@@ -184,7 +211,52 @@ AerospikeConstants aerospike_constants[] = {
 
 	{ AS_RECORD_DEFAULT_TTL                 ,   "TTL_NAMESPACE_DEFAULT"},
 	{ AS_RECORD_NO_EXPIRE_TTL               ,   "TTL_NEVER_EXPIRE"},
-	{ -2                                    ,   "TTL_DONT_UPDATE"}
+	{ AS_RECORD_NO_CHANGE_TTL               ,   "TTL_DONT_UPDATE"},
+	{ AS_AUTH_INTERNAL, "AUTH_INTERNAL"},
+	{ AS_AUTH_EXTERNAL, "AUTH_EXTERNAL"},
+	{ AS_AUTH_EXTERNAL_INSECURE, "AUTH_EXTERNAL_INSECURE"},
+	/* New CDT Operations, post 3.16.0.1 */
+	{OP_LIST_GET_BY_INDEX, "OP_LIST_GET_BY_INDEX"},
+	{OP_LIST_GET_BY_INDEX_RANGE, "OP_LIST_GET_BY_INDEX_RANGE"},
+	{OP_LIST_GET_BY_RANK, "OP_LIST_GET_BY_RANK"},
+	{OP_LIST_GET_BY_RANK_RANGE, "OP_LIST_GET_BY_RANK_RANGE"},
+	{OP_LIST_GET_BY_VALUE, "OP_LIST_GET_BY_VALUE"},
+	{OP_LIST_GET_BY_VALUE_LIST, "OP_LIST_GET_BY_VALUE_LIST"},
+	{OP_LIST_GET_BY_VALUE_RANGE, "OP_LIST_GET_BY_VALUE_RANGE"},
+	{OP_LIST_REMOVE_BY_INDEX, "OP_LIST_REMOVE_BY_INDEX"},
+	{OP_LIST_REMOVE_BY_INDEX_RANGE, "OP_LIST_REMOVE_BY_INDEX_RANGE"},
+	{OP_LIST_REMOVE_BY_RANK, "OP_LIST_REMOVE_BY_RANK"},
+	{OP_LIST_REMOVE_BY_RANK_RANGE, "OP_LIST_REMOVE_BY_RANK_RANGE"},
+	{OP_LIST_REMOVE_BY_VALUE, "OP_LIST_REMOVE_BY_VALUE"},
+	{OP_LIST_REMOVE_BY_VALUE_LIST, "OP_LIST_REMOVE_BY_VALUE_LIST"},
+	{OP_LIST_REMOVE_BY_VALUE_RANGE, "OP_LIST_REMOVE_BY_VALUE_RANGE"},
+	{OP_LIST_SET_ORDER, "OP_LIST_SET_ORDER"},
+	{OP_LIST_SORT, "OP_LIST_SORT"},
+	{AS_LIST_RETURN_NONE, "LIST_RETURN_NONE"},
+	{AS_LIST_RETURN_INDEX, "LIST_RETURN_INDEX"},
+	{AS_LIST_RETURN_REVERSE_INDEX, "LIST_RETURN_REVERSE_INDEX"},
+	{AS_LIST_RETURN_RANK, "LIST_RETURN_RANK"},
+	{AS_LIST_RETURN_REVERSE_RANK, "LIST_RETURN_REVERSE_RANK"},
+	{AS_LIST_RETURN_COUNT, "LIST_RETURN_COUNT"},
+	{AS_LIST_RETURN_VALUE, "LIST_RETURN_VALUE"},
+	{AS_LIST_SORT_DROP_DUPLICATES, "LIST_SORT_DROP_DUPLICATES"},
+	{AS_LIST_SORT_DEFAULT, "LIST_SORT_DEFAULT"},
+	{AS_LIST_WRITE_ADD_UNIQUE, "LIST_WRITE_ADD_UNIQUE"},
+	{AS_LIST_WRITE_INSERT_BOUNDED, "LIST_WRITE_INSERT_BOUNDED"},
+	{AS_LIST_ORDERED, "LIST_ORDERED"},
+	{AS_LIST_UNORDERED, "LIST_UNORDERED"},
+	{OP_LIST_REMOVE_BY_VALUE_RANK_RANGE_REL, "OP_LIST_REMOVE_BY_VALUE_RANK_RANGE_REL"},
+	{OP_LIST_GET_BY_VALUE_RANK_RANGE_REL, "OP_LIST_GET_BY_VALUE_RANK_RANGE_REL"},
+	{OP_MAP_REMOVE_BY_VALUE_RANK_RANGE_REL, "OP_MAP_REMOVE_BY_VALUE_RANK_RANGE_REL"},
+	{OP_MAP_REMOVE_BY_KEY_INDEX_RANGE_REL, "OP_MAP_REMOVE_BY_KEY_INDEX_RANGE_REL"},
+	{OP_MAP_GET_BY_VALUE_RANK_RANGE_REL, "OP_MAP_GET_BY_VALUE_RANK_RANGE_REL"},
+	{OP_MAP_GET_BY_KEY_INDEX_RANGE_REL, "OP_MAP_GET_BY_KEY_INDEX_RANGE_REL" },
+
+	{ AS_MAP_WRITE_NO_FAIL, "MAP_WRITE_NO_FAIL"},
+	{ AS_MAP_WRITE_PARTIAL, "MAP_WRITE_PARTIAL"},
+
+	{ AS_LIST_WRITE_NO_FAIL, "LIST_WRITE_NO_FAIL"},
+	{ AS_LIST_WRITE_PARTIAL, "LIST_WRITE_PARTIAL"}
 };
 
 static
@@ -263,17 +335,6 @@ void set_scan_options(as_error *err, as_scan* scan_p, PyObject * py_options)
 					as_error_update(err, AEROSPIKE_ERR_PARAM, "Unable to set scan nobins");
 					break;
 				}
-			} else if (strcmp("include_ldt", key_name) == 0) {
-				if (!PyBool_Check(value)) {
-					as_error_update(err, AEROSPIKE_ERR_PARAM, "Invalid value(type) for nobins");
-					break;
-				}
-				val = (int8_t)PyObject_IsTrue(value);
-				if (val == -1) {
-					as_error_update(err, AEROSPIKE_ERR_PARAM, "Unable to set scan nobins");
-					break;
-				}
-				scan_p->include_ldt = val;
 			} else {
 				as_error_update(err, AEROSPIKE_ERR_PARAM, "Invalid value for scan options");
 				break;
@@ -284,6 +345,26 @@ void set_scan_options(as_error *err, as_scan* scan_p, PyObject * py_options)
 	}
 }
 
+as_status set_query_options(as_error* err, PyObject* query_options, as_query* query) {
+	PyObject* no_bins_val = NULL;
+	if (!query_options || query_options == Py_None) {
+		return AEROSPIKE_OK;
+	}
+
+	if (!PyDict_Check(query_options)) {
+		return as_error_update(err, AEROSPIKE_ERR_PARAM, "query options must be a dictionary");
+	}
+
+	no_bins_val = PyDict_GetItemString(query_options, "nobins");
+	if (no_bins_val) {
+		if (!PyBool_Check(no_bins_val)) {
+			return as_error_update(err, AEROSPIKE_ERR_PARAM, "nobins value must be a bool");
+		}
+		query->no_bins = PyObject_IsTrue(no_bins_val);
+	}
+	return AEROSPIKE_OK;
+
+}
 /**
  * Declares policy constants.
  */
@@ -356,10 +437,19 @@ as_status pyobject_to_policy_apply(as_error * err, PyObject * py_policy,
 	as_policy_apply_copy(config_apply_policy, policy);
 
 	// Set policy fields
-	POLICY_SET_FIELD(timeout, uint32_t);
+	POLICY_SET_TOTAL_TIMEOUT_FROM_TIMEOUT();
+
+	POLICY_SET_BASE_FIELD(total_timeout, uint32_t);
+	POLICY_SET_BASE_FIELD(socket_timeout, uint32_t);
+	POLICY_SET_BASE_FIELD(max_retries, uint32_t);
+	POLICY_SET_BASE_FIELD(sleep_between_retries, uint32_t);
+
 	POLICY_SET_FIELD(key, as_policy_key);
-	POLICY_SET_FIELD(retry_on_timeout, bool);
+	POLICY_SET_FIELD(replica, as_policy_replica);
+	POLICY_SET_FIELD(gen, as_policy_gen);
+	POLICY_SET_FIELD(commit_level, as_policy_commit_level);
 	POLICY_SET_FIELD(durable_delete, bool);
+	POLICY_SET_FIELD(linearize_read, bool);
 
 	// Update the policy
 	POLICY_UPDATE();
@@ -413,8 +503,15 @@ as_status pyobject_to_policy_query(as_error * err, PyObject * py_policy,
 	as_policy_query_copy(config_query_policy, policy);
 
 	// Set policy fields
-	POLICY_SET_FIELD(timeout, uint32_t);
+	POLICY_SET_TOTAL_TIMEOUT_FROM_TIMEOUT();
+	POLICY_SET_BASE_FIELD(total_timeout, uint32_t);
+	POLICY_SET_BASE_FIELD(socket_timeout, uint32_t);
+	POLICY_SET_BASE_FIELD(max_retries, uint32_t);
+	POLICY_SET_BASE_FIELD(sleep_between_retries, uint32_t);
 
+
+	POLICY_SET_FIELD(deserialize, bool);
+	POLICY_SET_FIELD(fail_on_cluster_change, bool);
 	// Update the policy
 	POLICY_UPDATE();
 
@@ -439,11 +536,18 @@ as_status pyobject_to_policy_read(as_error * err, PyObject * py_policy,
 	as_policy_read_copy(config_read_policy, policy);
 
 	// Set policy fields
-	POLICY_SET_FIELD(timeout, uint32_t);
+	POLICY_SET_TOTAL_TIMEOUT_FROM_TIMEOUT();
+
+	POLICY_SET_BASE_FIELD(total_timeout, uint32_t);
+	POLICY_SET_BASE_FIELD(socket_timeout, uint32_t);
+	POLICY_SET_BASE_FIELD(max_retries, uint32_t);
+	POLICY_SET_BASE_FIELD(sleep_between_retries, uint32_t);
+
 	POLICY_SET_FIELD(key, as_policy_key);
 	POLICY_SET_FIELD(consistency_level, as_policy_consistency_level);
 	POLICY_SET_FIELD(replica, as_policy_replica);
-	POLICY_SET_FIELD(retry_on_timeout, bool);
+	POLICY_SET_FIELD(deserialize, bool);
+	POLICY_SET_FIELD(linearize_read, bool);
 
 	// Update the policy
 	POLICY_UPDATE();
@@ -469,13 +573,19 @@ as_status pyobject_to_policy_remove(as_error * err, PyObject * py_policy,
 	as_policy_remove_copy(config_remove_policy, policy);
 
 	// Set policy fields
-	POLICY_SET_FIELD(timeout, uint32_t);
+	POLICY_SET_TOTAL_TIMEOUT_FROM_TIMEOUT();
+
+	POLICY_SET_BASE_FIELD(total_timeout, uint32_t);
+	POLICY_SET_BASE_FIELD(socket_timeout, uint32_t);
+	POLICY_SET_BASE_FIELD(max_retries, uint32_t);
+	POLICY_SET_BASE_FIELD(sleep_between_retries, uint32_t);
+
 	POLICY_SET_FIELD(generation, uint16_t);
-	POLICY_SET_FIELD(retry, as_policy_retry);
+
 	POLICY_SET_FIELD(key, as_policy_key);
 	POLICY_SET_FIELD(gen, as_policy_gen);
 	POLICY_SET_FIELD(commit_level, as_policy_commit_level);
-	POLICY_SET_FIELD(retry_on_timeout, bool);
+	POLICY_SET_FIELD(replica, as_policy_replica);
 	POLICY_SET_FIELD(durable_delete, bool);
 
 	// Update the policy
@@ -503,8 +613,13 @@ as_status pyobject_to_policy_scan(as_error * err, PyObject * py_policy,
 
 	// Set policy fields
 	// server side socket_timeout
-	POLICY_SET_FIELD(socket_timeout, uint32_t);
-	POLICY_SET_FIELD(timeout, uint32_t);
+	POLICY_SET_TOTAL_TIMEOUT_FROM_TIMEOUT();
+
+	POLICY_SET_BASE_FIELD(total_timeout, uint32_t);
+	POLICY_SET_BASE_FIELD(socket_timeout, uint32_t);
+	POLICY_SET_BASE_FIELD(max_retries, uint32_t);
+	POLICY_SET_BASE_FIELD(sleep_between_retries, uint32_t);
+
 	POLICY_SET_FIELD(fail_on_cluster_change, bool);
 	POLICY_SET_FIELD(durable_delete, bool);
 
@@ -532,14 +647,21 @@ as_status pyobject_to_policy_write(as_error * err, PyObject * py_policy,
 	as_policy_write_copy(config_write_policy, policy);
 
 	// Set policy fields
-	POLICY_SET_FIELD(timeout, uint32_t);
-	POLICY_SET_FIELD(retry, as_policy_retry);
+	POLICY_SET_TOTAL_TIMEOUT_FROM_TIMEOUT();
+
+	// Base policy_fields
+	POLICY_SET_BASE_FIELD(total_timeout, uint32_t);
+	POLICY_SET_BASE_FIELD(socket_timeout, uint32_t);
+	POLICY_SET_BASE_FIELD(max_retries, uint32_t);
+	POLICY_SET_BASE_FIELD(sleep_between_retries, uint32_t);
+
 	POLICY_SET_FIELD(key, as_policy_key);
 	POLICY_SET_FIELD(gen, as_policy_gen);
 	POLICY_SET_FIELD(exists, as_policy_exists);
 	POLICY_SET_FIELD(commit_level, as_policy_commit_level);
-	POLICY_SET_FIELD(retry_on_timeout, bool);
 	POLICY_SET_FIELD(durable_delete, bool);
+	POLICY_SET_FIELD(replica, as_policy_replica);
+	POLICY_SET_FIELD(compression_threshold, uint32_t);
 
 	// Update the policy
 	POLICY_UPDATE();
@@ -565,15 +687,22 @@ as_status pyobject_to_policy_operate(as_error * err, PyObject * py_policy,
 	as_policy_operate_copy(config_operate_policy, policy);
 
 	// Set policy fields
-	POLICY_SET_FIELD(timeout, uint32_t);
-	POLICY_SET_FIELD(retry, as_policy_retry);
+	POLICY_SET_TOTAL_TIMEOUT_FROM_TIMEOUT();
+
+	POLICY_SET_BASE_FIELD(total_timeout, uint32_t);
+	POLICY_SET_BASE_FIELD(socket_timeout, uint32_t);
+	POLICY_SET_BASE_FIELD(max_retries, uint32_t);
+	POLICY_SET_BASE_FIELD(sleep_between_retries, uint32_t);
+
 	POLICY_SET_FIELD(key, as_policy_key);
 	POLICY_SET_FIELD(gen, as_policy_gen);
 	POLICY_SET_FIELD(commit_level, as_policy_commit_level);
 	POLICY_SET_FIELD(consistency_level, as_policy_consistency_level);
 	POLICY_SET_FIELD(replica, as_policy_replica);
-	POLICY_SET_FIELD(retry_on_timeout, bool);
 	POLICY_SET_FIELD(durable_delete, bool);
+	POLICY_SET_FIELD(deserialize, bool);
+	POLICY_SET_FIELD(linearize_read, bool);
+	POLICY_SET_FIELD(exists, as_policy_exists);
 
 	// Update the policy
 	POLICY_UPDATE();
@@ -599,8 +728,20 @@ as_status pyobject_to_policy_batch(as_error * err, PyObject * py_policy,
 	as_policy_batch_copy(config_batch_policy, policy);
 
 	// Set policy fields
-	POLICY_SET_FIELD(timeout, uint32_t);
-	POLICY_SET_FIELD(retry_on_timeout, bool);
+	POLICY_SET_TOTAL_TIMEOUT_FROM_TIMEOUT();
+
+	POLICY_SET_BASE_FIELD(total_timeout, uint32_t);
+	POLICY_SET_BASE_FIELD(socket_timeout, uint32_t);
+	POLICY_SET_BASE_FIELD(max_retries, uint32_t);
+	POLICY_SET_BASE_FIELD(sleep_between_retries, uint32_t);
+
+	POLICY_SET_FIELD(consistency_level, as_policy_consistency_level);
+	POLICY_SET_FIELD(concurrent, bool);
+	POLICY_SET_FIELD(use_batch_direct, bool);
+	POLICY_SET_FIELD(allow_inline, bool);
+	POLICY_SET_FIELD(send_set_name, bool);
+	POLICY_SET_FIELD(deserialize, bool);
+	POLICY_SET_FIELD(linearize_read, bool);
 
 	// Update the policy
 	POLICY_UPDATE();
@@ -624,3 +765,64 @@ as_status pyobject_to_map_policy(as_error * err, PyObject * py_policy,
 
 	return err->code;
 }
+
+as_status
+pyobject_to_list_policy(as_error* err, PyObject* py_policy, as_list_policy* list_policy)
+{
+	as_list_policy_init(list_policy);
+	PyObject* py_val = NULL;
+	long list_order = AS_LIST_UNORDERED;
+	long flags = AS_LIST_WRITE_DEFAULT;
+
+	if (!py_policy || py_policy == Py_None) {
+		return AEROSPIKE_OK;
+	}
+
+	if (!PyDict_Check(py_policy)) {
+		return as_error_update(err, AEROSPIKE_ERR_PARAM, "List policy must be a dictionary.");
+	}
+
+	py_val = PyDict_GetItemString(py_policy, "list_order");
+    if (py_val && py_val != Py_None) {
+        if (PyInt_Check(py_val)) {
+            list_order = (int64_t)PyInt_AsLong(py_val);
+            if (PyErr_Occurred()) {
+                return as_error_update(err, AEROSPIKE_ERR_PARAM, "Failed to convert list_order");
+
+            }
+    	}
+        else if (PyLong_Check(py_val)) {
+        	list_order = (int64_t)PyLong_AsLong(py_val);
+            if (PyErr_Occurred()) {
+                return as_error_update(err, AEROSPIKE_ERR_PARAM, "Failed to convert list_order");
+            }
+        }
+        else {
+            return as_error_update(err, AEROSPIKE_ERR_PARAM, "Invalid List order");
+        }
+	}
+
+	py_val = PyDict_GetItemString(py_policy, "write_flags");
+    if (py_val && py_val != Py_None) {
+        if (PyInt_Check(py_val)) {
+        	flags = (int64_t)PyInt_AsLong(py_val);
+            if (PyErr_Occurred()) {
+                return as_error_update(err, AEROSPIKE_ERR_PARAM, "Failed to convert write_flags");
+
+            }
+    	}
+        else if (PyLong_Check(py_val)) {
+        	flags = (int64_t)PyLong_AsLong(py_val);
+            if (PyErr_Occurred()) {
+                return as_error_update(err, AEROSPIKE_ERR_PARAM, "Failed to convert write_flags");
+            }
+        }
+        else {
+            return as_error_update(err, AEROSPIKE_ERR_PARAM, "Invalid write_flags");
+        }
+	}
+
+	as_list_policy_set(list_policy, (as_list_order)list_order, (as_list_write_flags)flags);
+
+	return AEROSPIKE_OK;
+}
