@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2013-2016 Aerospike, Inc.
+ * Copyright 2013-2017 Aerospike, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -110,12 +110,12 @@ PyObject * AerospikeQuery_Foreach(AerospikeQuery * self, PyObject * args, PyObje
 	// Python Function Arguments
 	PyObject * py_callback = NULL;
 	PyObject * py_policy = NULL;
-
+	PyObject * py_options = NULL;
 	// Python Function Keyword Arguments
-	static char * kwlist[] = {"callback", "policy", NULL};
+	static char * kwlist[] = {"callback", "policy", "options", NULL};
 
 	// Python Function Argument Parsing
-	if (PyArg_ParseTupleAndKeywords(args, kwds, "O|O:foreach", kwlist, &py_callback, &py_policy) == false) {
+	if (PyArg_ParseTupleAndKeywords(args, kwds, "O|OO:foreach", kwlist, &py_callback, &py_policy, &py_options) == false) {
 		as_query_destroy(&self->query);
 		return NULL;
 	}
@@ -151,6 +151,10 @@ PyObject * AerospikeQuery_Foreach(AerospikeQuery * self, PyObject * args, PyObje
 		goto CLEANUP;
 	}
 
+	if (set_query_options(&err, py_options, &self->query) != AEROSPIKE_OK) {
+		goto CLEANUP;
+	}
+
 	// We are spawning multiple threads
 	PyThreadState * _save = PyEval_SaveThread();
 
